moody.m.erc20 API documentation
Description: Generated wrapper for ERC20 Solidity contract.

# Module `moody.m.erc20`

Generated wrapper for ERC20 Solidity contract.

Expand source code

    """Generated wrapper for ERC20 Solidity contract."""

    # pylint: disable=too-many-arguments

    import json
    from typing import (  # pylint: disable=unused-import
        Any,
        List,
        Optional,
        Tuple,
        Union,
    )

    from eth_utils import to_checksum_address
    from mypy_extensions import TypedDict  # pylint: disable=unused-import
    from hexbytes import HexBytes
    from web3 import Web3
    from web3.contract import ContractFunction
    from web3.datastructures import AttributeDict
    from web3.providers.base import BaseProvider

    from ..bases import ContractMethod, Validator
    from ..tx_params import TxParams

    # Try to import a custom validator class definition; if there isn't one,
    # declare one that we can instantiate for the default argument to the
    # constructor for ERC20 below.
    try:
        # both mypy and pylint complain about what we're doing here, but this
        # works just fine, so their messages have been disabled here.
        from . import (  # type: ignore # pylint: disable=import-self
            ERC20Validator,
        )
    except ImportError:

        class ERC20Validator(  # type: ignore
            Validator
        ):
            """No-op input validator."""

    try:
        from .middleware import MIDDLEWARE  # type: ignore
    except ImportError:
        pass

    class AllowanceMethod(ContractMethod): # pylint: disable=invalid-name
        """Various interfaces to the allowance method."""

        def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
            """Persist instance data."""
            super().__init__(web3_or_provider, contract_address, validator)
            self._underlying_method = contract_function

        def validate_and_normalize_inputs(self, owner: str, spender: str):
            """Validate the inputs to the allowance method."""
            self.validator.assert_valid(
                method_name='allowance',
                parameter_name='owner',
                argument_value=owner,
            )
            owner = self.validate_and_checksum_address(owner)
            self.validator.assert_valid(
                method_name='allowance',
                parameter_name='spender',
                argument_value=spender,
            )
            spender = self.validate_and_checksum_address(spender)
            return (owner, spender)

        def block_call(self, owner: str, spender: str, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters

            """
            (owner, spender) = self.validate_and_normalize_inputs(owner, spender)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(owner, spender).call(tx_params.as_dict())
            return int(returned)

        def estimate_gas(self, owner: str, spender: str, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (owner, spender) = self.validate_and_normalize_inputs(owner, spender)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(owner, spender).estimateGas(tx_params.as_dict())

    class ApproveMethod(ContractMethod): # pylint: disable=invalid-name
        """Various interfaces to the approve method."""

        def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
            """Persist instance data."""
            super().__init__(web3_or_provider, contract_address, validator)
            self._underlying_method = contract_function

        def validate_and_normalize_inputs(self, spender: str, amount: int):
            """Validate the inputs to the approve method."""
            self.validator.assert_valid(
                method_name='approve',
                parameter_name='spender',
                argument_value=spender,
            )
            spender = self.validate_and_checksum_address(spender)
            self.validator.assert_valid(
                method_name='approve',
                parameter_name='amount',
                argument_value=amount,
            )
            # safeguard against fractional inputs
            amount = int(amount)
            return (spender, amount)

        def block_call(self, spender: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (spender, amount) = self.validate_and_normalize_inputs(spender, amount)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(spender, amount).call(tx_params.as_dict())
            return bool(returned)

        def send_transaction(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, amount).transact(tx_params.as_dict())

        def build_transaction(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, amount).buildTransaction(tx_params.as_dict())

        def estimate_gas(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, amount).estimateGas(tx_params.as_dict())

    class BalanceOfMethod(ContractMethod): # pylint: disable=invalid-name
        """Various interfaces to the balanceOf method."""

        def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
            """Persist instance data."""
            super().__init__(web3_or_provider, contract_address, validator)
            self._underlying_method = contract_function

        def validate_and_normalize_inputs(self, account: str):
            """Validate the inputs to the balanceOf method."""
            self.validator.assert_valid(
                method_name='balanceOf',
                parameter_name='account',
                argument_value=account,
            )
            account = self.validate_and_checksum_address(account)
            return (account)

        def block_call(self, account: str, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters

            """
            (account) = self.validate_and_normalize_inputs(account)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(account).call(tx_params.as_dict())
            return int(returned)

        def estimate_gas(self, account: str, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (account) = self.validate_and_normalize_inputs(account)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(account).estimateGas(tx_params.as_dict())

    class DecreaseAllowanceMethod(ContractMethod): # pylint: disable=invalid-name
        """Various interfaces to the decreaseAllowance method."""

        def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
            """Persist instance data."""
            super().__init__(web3_or_provider, contract_address, validator)
            self._underlying_method = contract_function

        def validate_and_normalize_inputs(self, spender: str, subtracted_value: int):
            """Validate the inputs to the decreaseAllowance method."""
            self.validator.assert_valid(
                method_name='decreaseAllowance',
                parameter_name='spender',
                argument_value=spender,
            )
            spender = self.validate_and_checksum_address(spender)
            self.validator.assert_valid(
                method_name='decreaseAllowance',
                parameter_name='subtractedValue',
                argument_value=subtracted_value,
            )
            # safeguard against fractional inputs
            subtracted_value = int(subtracted_value)
            return (spender, subtracted_value)

        def block_call(self, spender: str, subtracted_value: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(spender, subtracted_value).call(tx_params.as_dict())
            return bool(returned)

        def send_transaction(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, subtracted_value).transact(tx_params.as_dict())

        def build_transaction(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, subtracted_value).buildTransaction(tx_params.as_dict())

        def estimate_gas(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, subtracted_value).estimateGas(tx_params.as_dict())

    class IncreaseAllowanceMethod(ContractMethod): # pylint: disable=invalid-name
        """Various interfaces to the increaseAllowance method."""

        def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
            """Persist instance data."""
            super().__init__(web3_or_provider, contract_address, validator)
            self._underlying_method = contract_function

        def validate_and_normalize_inputs(self, spender: str, added_value: int):
            """Validate the inputs to the increaseAllowance method."""
            self.validator.assert_valid(
                method_name='increaseAllowance',
                parameter_name='spender',
                argument_value=spender,
            )
            spender = self.validate_and_checksum_address(spender)
            self.validator.assert_valid(
                method_name='increaseAllowance',
                parameter_name='addedValue',
                argument_value=added_value,
            )
            # safeguard against fractional inputs
            added_value = int(added_value)
            return (spender, added_value)

        def block_call(self, spender: str, added_value: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(spender, added_value).call(tx_params.as_dict())
            return bool(returned)

        def send_transaction(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, added_value).transact(tx_params.as_dict())

        def build_transaction(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, added_value).buildTransaction(tx_params.as_dict())

        def estimate_gas(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, added_value).estimateGas(tx_params.as_dict())

    class TotalSupplyMethod(ContractMethod): # pylint: disable=invalid-name
        """Various interfaces to the totalSupply method."""

        def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction):
            """Persist instance data."""
            super().__init__(web3_or_provider, contract_address)
            self._underlying_method = contract_function

        def block_call(self, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters

            """

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method().call(tx_params.as_dict())
            return int(returned)

        def estimate_gas(self, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method().estimateGas(tx_params.as_dict())

    class TransferMethod(ContractMethod): # pylint: disable=invalid-name
        """Various interfaces to the transfer method."""

        def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
            """Persist instance data."""
            super().__init__(web3_or_provider, contract_address, validator)
            self._underlying_method = contract_function

        def validate_and_normalize_inputs(self, recipient: str, amount: int):
            """Validate the inputs to the transfer method."""
            self.validator.assert_valid(
                method_name='transfer',
                parameter_name='recipient',
                argument_value=recipient,
            )
            recipient = self.validate_and_checksum_address(recipient)
            self.validator.assert_valid(
                method_name='transfer',
                parameter_name='amount',
                argument_value=amount,
            )
            # safeguard against fractional inputs
            amount = int(amount)
            return (recipient, amount)

        def block_call(self, recipient: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(recipient, amount).call(tx_params.as_dict())
            return bool(returned)

        def send_transaction(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(recipient, amount).transact(tx_params.as_dict())

        def build_transaction(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(recipient, amount).buildTransaction(tx_params.as_dict())

        def estimate_gas(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(recipient, amount).estimateGas(tx_params.as_dict())

    class TransferFromMethod(ContractMethod): # pylint: disable=invalid-name
        """Various interfaces to the transferFrom method."""

        def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
            """Persist instance data."""
            super().__init__(web3_or_provider, contract_address, validator)
            self._underlying_method = contract_function

        def validate_and_normalize_inputs(self, sender: str, recipient: str, amount: int):
            """Validate the inputs to the transferFrom method."""
            self.validator.assert_valid(
                method_name='transferFrom',
                parameter_name='sender',
                argument_value=sender,
            )
            sender = self.validate_and_checksum_address(sender)
            self.validator.assert_valid(
                method_name='transferFrom',
                parameter_name='recipient',
                argument_value=recipient,
            )
            recipient = self.validate_and_checksum_address(recipient)
            self.validator.assert_valid(
                method_name='transferFrom',
                parameter_name='amount',
                argument_value=amount,
            )
            # safeguard against fractional inputs
            amount = int(amount)
            return (sender, recipient, amount)

        def block_call(self, sender: str, recipient: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(sender, recipient, amount).call(tx_params.as_dict())
            return bool(returned)

        def send_transaction(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(sender, recipient, amount).transact(tx_params.as_dict())

        def build_transaction(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(sender, recipient, amount).buildTransaction(tx_params.as_dict())

        def estimate_gas(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(sender, recipient, amount).estimateGas(tx_params.as_dict())

    # pylint: disable=too-many-public-methods,too-many-instance-attributes
    class ERC20:
        """Wrapper class for ERC20 Solidity contract."""
        _fn_allowance: AllowanceMethod
        """Constructor-initialized instance of
        :class:`AllowanceMethod`.
        """

        _fn_approve: ApproveMethod
        """Constructor-initialized instance of
        :class:`ApproveMethod`.
        """

        _fn_balance_of: BalanceOfMethod
        """Constructor-initialized instance of
        :class:`BalanceOfMethod`.
        """

        _fn_decrease_allowance: DecreaseAllowanceMethod
        """Constructor-initialized instance of
        :class:`DecreaseAllowanceMethod`.
        """

        _fn_increase_allowance: IncreaseAllowanceMethod
        """Constructor-initialized instance of
        :class:`IncreaseAllowanceMethod`.
        """

        _fn_total_supply: TotalSupplyMethod
        """Constructor-initialized instance of
        :class:`TotalSupplyMethod`.
        """

        _fn_transfer: TransferMethod
        """Constructor-initialized instance of
        :class:`TransferMethod`.
        """

        _fn_transfer_from: TransferFromMethod
        """Constructor-initialized instance of
        :class:`TransferFromMethod`.
        """

        def __init__(
            self,
            web3_or_provider: Union[Web3, BaseProvider],
            contract_address: str,
            validator: ERC20Validator = None,
        ):
            """Get an instance of wrapper for smart contract.

            :param web3_or_provider: Either an instance of `web3.Web3`:code: or
                `web3.providers.base.BaseProvider`:code:
            :param contract_address: where the contract has been deployed
            :param validator: for validation of method inputs.
            """
            # pylint: disable=too-many-statements

            self.contract_address = contract_address

            if not validator:
                validator = ERC20Validator(web3_or_provider, contract_address)

            web3 = None
            if isinstance(web3_or_provider, BaseProvider):
                web3 = Web3(web3_or_provider)
            elif isinstance(web3_or_provider, Web3):
                web3 = web3_or_provider
            else:
                raise TypeError(
                    "Expected parameter 'web3_or_provider' to be an instance of either"
                    + " Web3 or BaseProvider"
                )

            # if any middleware was imported, inject it
            try:
                MIDDLEWARE
            except NameError:
                pass
            else:
                try:
                    for middleware in MIDDLEWARE:
                        web3.middleware_onion.inject(
                             middleware['function'], layer=middleware['layer'],
                        )
                except ValueError as value_error:
                    if value_error.args == ("You can't add the same un-named instance twice",):
                        pass

            self._web3_eth = web3.eth

            functions = self._web3_eth.contract(address=to_checksum_address(contract_address), abi=ERC20.abi()).functions

            self.call_contract_fee_amount:int = 1000000
            self.call_contract_debug_flag:bool = False

            self._fn_allowance = AllowanceMethod(web3_or_provider, contract_address, functions.allowance, validator)

            self._fn_approve = ApproveMethod(web3_or_provider, contract_address, functions.approve, validator)

            self._fn_balance_of = BalanceOfMethod(web3_or_provider, contract_address, functions.balanceOf, validator)

            self._fn_decrease_allowance = DecreaseAllowanceMethod(web3_or_provider, contract_address, functions.decreaseAllowance, validator)

            self._fn_increase_allowance = IncreaseAllowanceMethod(web3_or_provider, contract_address, functions.increaseAllowance, validator)

            self._fn_total_supply = TotalSupplyMethod(web3_or_provider, contract_address, functions.totalSupply)

            self._fn_transfer = TransferMethod(web3_or_provider, contract_address, functions.transfer, validator)

            self._fn_transfer_from = TransferFromMethod(web3_or_provider, contract_address, functions.transferFrom, validator)

        """
        Implementation of event approval in contract ERC20
        Get log entry for Approval event.
                :param tx_hash: hash of transaction emitting Approval event
        """

        def event_approval(
                self, tx_hash: Union[HexBytes, bytes]
        ) -> Tuple[AttributeDict]:
            tx_receipt = self._web3_eth.getTransactionReceipt(tx_hash)
            return self._web3_eth.contract(address=to_checksum_address(self.contract_address), abi=ERC20.abi()).events.Approval().processReceipt(tx_receipt)
        """
        Implementation of event transfer in contract ERC20
        Get log entry for Transfer event.
                :param tx_hash: hash of transaction emitting Transfer event
        """

        def event_transfer(
                self, tx_hash: Union[HexBytes, bytes]
        ) -> Tuple[AttributeDict]:
            tx_receipt = self._web3_eth.getTransactionReceipt(tx_hash)
            return self._web3_eth.contract(address=to_checksum_address(self.contract_address), abi=ERC20.abi()).events.Transfer().processReceipt(tx_receipt)

        """
        implementation of allowance in contract ERC20

        """

        def allowance(self,owner: str, spender: str) -> int:
            return self._fn_allowance.block_call(owner, spender)
        """
        implementation of approve in contract ERC20

        """

        def approve(self,spender: str, amount: int) -> bool:
            return self._fn_approve.block_call(spender, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)
        """
        implementation of balance_of in contract ERC20

        """

        def balance_of(self,account: str) -> int:
            return self._fn_balance_of.block_call(account)
        """
        implementation of decrease_allowance in contract ERC20

        """

        def decrease_allowance(self,spender: str, subtracted_value: int) -> bool:
            return self._fn_decrease_allowance.block_call(spender, subtracted_value, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)
        """
        implementation of increase_allowance in contract ERC20

        """

        def increase_allowance(self,spender: str, added_value: int) -> bool:
            return self._fn_increase_allowance.block_call(spender, added_value, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)
        """
        implementation of total_supply in contract ERC20

        """

        def total_supply(self) -> int:
            return self._fn_total_supply.block_call()
        """
        implementation of transfer in contract ERC20

        """

        def transfer(self,recipient: str, amount: int) -> bool:
            return self._fn_transfer.block_call(recipient, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)
        """
        implementation of transfer_from in contract ERC20

        """

        def transfer_from(self,sender: str, recipient: str, amount: int) -> bool:
            return self._fn_transfer_from.block_call(sender, recipient, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)

        def CallContractFee(self, amount:int)-> "ERC20":
            self.call_contract_fee_amount = amount
            return self

        def CallDebug(self, yesno: bool) -> "ERC20":
            self.call_contract_debug_flag = yesno
            return self

        @staticmethod
        def abi():
            """Return the ABI to the underlying contract."""
            return json.loads(
                '[{"anonymous":false,"inputs":[{"indexed":true,"internalType":"address","name":"owner","type":"address"},{"indexed":true,"internalType":"address","name":"spender","type":"address"},{"indexed":false,"internalType":"uint256","name":"value","type":"uint256"}],"name":"Approval","type":"event"},{"anonymous":false,"inputs":[{"indexed":true,"internalType":"address","name":"from","type":"address"},{"indexed":true,"internalType":"address","name":"to","type":"address"},{"indexed":false,"internalType":"uint256","name":"value","type":"uint256"}],"name":"Transfer","type":"event"},{"constant":true,"inputs":[{"internalType":"address","name":"owner","type":"address"},{"internalType":"address","name":"spender","type":"address"}],"name":"allowance","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"approve","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[{"internalType":"address","name":"account","type":"address"}],"name":"balanceOf","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"subtractedValue","type":"uint256"}],"name":"decreaseAllowance","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"addedValue","type":"uint256"}],"name":"increaseAllowance","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[],"name":"totalSupply","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"recipient","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"transfer","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"sender","type":"address"},{"internalType":"address","name":"recipient","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"transferFrom","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"}]'  # noqa: E501 (line-too-long)
            )

    # pylint: disable=too-many-lines

## Classes

`class AllowanceMethod (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, contract_function: web3.contract.ContractFunction, validator: Validator = None)`

  Various interfaces to the allowance method.

  Persist instance data.

  Expand source code

      class AllowanceMethod(ContractMethod): # pylint: disable=invalid-name
          """Various interfaces to the allowance method."""

          def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
              """Persist instance data."""
              super().__init__(web3_or_provider, contract_address, validator)
              self._underlying_method = contract_function

          def validate_and_normalize_inputs(self, owner: str, spender: str):
              """Validate the inputs to the allowance method."""
              self.validator.assert_valid(
                  method_name='allowance',
                  parameter_name='owner',
                  argument_value=owner,
              )
              owner = self.validate_and_checksum_address(owner)
              self.validator.assert_valid(
                  method_name='allowance',
                  parameter_name='spender',
                  argument_value=spender,
              )
              spender = self.validate_and_checksum_address(spender)
              return (owner, spender)

          def block_call(self, owner: str, spender: str, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
              """Execute underlying contract method via eth_call.

              :param tx_params: transaction parameters

              """
              (owner, spender) = self.validate_and_normalize_inputs(owner, spender)

              tx_params: Optional[TxParams] = None
              tx_params = super().normalize_tx_params(tx_params)
              returned = self._underlying_method(owner, spender).call(tx_params.as_dict())
              return int(returned)

          def estimate_gas(self, owner: str, spender: str, tx_params: Optional[TxParams] = None) -> int:
              """Estimate gas consumption of method call."""
              (owner, spender) = self.validate_and_normalize_inputs(owner, spender)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(owner, spender).estimateGas(tx_params.as_dict())

  ### Ancestors

  - ContractMethod

  ### Methods

  `def block_call(self, owner: str, spender: str, val: int = 0, fee: int = 1000000, debug: bool = False) ‑> int`

    Execute underlying contract method via eth_call.

    :param tx_params: transaction parameters

    Expand source code

        def block_call(self, owner: str, spender: str, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters

            """
            (owner, spender) = self.validate_and_normalize_inputs(owner, spender)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(owner, spender).call(tx_params.as_dict())
            return int(returned)

  `def estimate_gas(self, owner: str, spender: str, tx_params: Union[TxParams, NoneType] = None) ‑> int`

    Estimate gas consumption of method call.

    Expand source code

        def estimate_gas(self, owner: str, spender: str, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (owner, spender) = self.validate_and_normalize_inputs(owner, spender)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(owner, spender).estimateGas(tx_params.as_dict())

  `def validate_and_normalize_inputs(self, owner: str, spender: str)`

    Validate the inputs to the allowance method.

    Expand source code

        def validate_and_normalize_inputs(self, owner: str, spender: str):
            """Validate the inputs to the allowance method."""
            self.validator.assert_valid(
                method_name='allowance',
                parameter_name='owner',
                argument_value=owner,
            )
            owner = self.validate_and_checksum_address(owner)
            self.validator.assert_valid(
                method_name='allowance',
                parameter_name='spender',
                argument_value=spender,
            )
            spender = self.validate_and_checksum_address(spender)
            return (owner, spender)

  ### Inherited members

  - `ContractMethod`:
    - `normalize_tx_params`
    - `validate_and_checksum_address`
`class ApproveMethod (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, contract_function: web3.contract.ContractFunction, validator: Validator = None)`

  Various interfaces to the approve method.

  Persist instance data.

  Expand source code

      class ApproveMethod(ContractMethod): # pylint: disable=invalid-name
          """Various interfaces to the approve method."""

          def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
              """Persist instance data."""
              super().__init__(web3_or_provider, contract_address, validator)
              self._underlying_method = contract_function

          def validate_and_normalize_inputs(self, spender: str, amount: int):
              """Validate the inputs to the approve method."""
              self.validator.assert_valid(
                  method_name='approve',
                  parameter_name='spender',
                  argument_value=spender,
              )
              spender = self.validate_and_checksum_address(spender)
              self.validator.assert_valid(
                  method_name='approve',
                  parameter_name='amount',
                  argument_value=amount,
              )
              # safeguard against fractional inputs
              amount = int(amount)
              return (spender, amount)

          def block_call(self, spender: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
              """Execute underlying contract method via eth_call.

              :param tx_params: transaction parameters
              :returns: the return value of the underlying method.
              """
              (spender, amount) = self.validate_and_normalize_inputs(spender, amount)

              tx_params: Optional[TxParams] = None
              tx_params = super().normalize_tx_params(tx_params)
              returned = self._underlying_method(spender, amount).call(tx_params.as_dict())
              return bool(returned)

          def send_transaction(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
              """Execute underlying contract method via eth_sendTransaction.

              :param tx_params: transaction parameters
              """
              (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, amount).transact(tx_params.as_dict())

          def build_transaction(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
              """Construct calldata to be used as input to the method."""
              (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, amount).buildTransaction(tx_params.as_dict())

          def estimate_gas(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
              """Estimate gas consumption of method call."""
              (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, amount).estimateGas(tx_params.as_dict())

  ### Ancestors

  - ContractMethod

  ### Methods

  `def block_call(self, spender: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) ‑> bool`

    Execute underlying contract method via eth_call.

    :param tx_params: transaction parameters :returns: the return value of the underlying method.

    Expand source code

        def block_call(self, spender: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (spender, amount) = self.validate_and_normalize_inputs(spender, amount)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(spender, amount).call(tx_params.as_dict())
            return bool(returned)

  `def build_transaction(self, spender: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> dict`

    Construct calldata to be used as input to the method.

    Expand source code

        def build_transaction(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, amount).buildTransaction(tx_params.as_dict())

  `def estimate_gas(self, spender: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> int`

    Estimate gas consumption of method call.

    Expand source code

        def estimate_gas(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, amount).estimateGas(tx_params.as_dict())

  `def send_transaction(self, spender: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> Union[hexbytes.main.HexBytes, bytes]`

    Execute underlying contract method via eth_sendTransaction.

    :param tx_params: transaction parameters

    Expand source code

        def send_transaction(self, spender: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (spender, amount) = self.validate_and_normalize_inputs(spender, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, amount).transact(tx_params.as_dict())

  `def validate_and_normalize_inputs(self, spender: str, amount: int)`

    Validate the inputs to the approve method.

    Expand source code

        def validate_and_normalize_inputs(self, spender: str, amount: int):
            """Validate the inputs to the approve method."""
            self.validator.assert_valid(
                method_name='approve',
                parameter_name='spender',
                argument_value=spender,
            )
            spender = self.validate_and_checksum_address(spender)
            self.validator.assert_valid(
                method_name='approve',
                parameter_name='amount',
                argument_value=amount,
            )
            # safeguard against fractional inputs
            amount = int(amount)
            return (spender, amount)

  ### Inherited members

  - `ContractMethod`:
    - `normalize_tx_params`
    - `validate_and_checksum_address`
`class BalanceOfMethod (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, contract_function: web3.contract.ContractFunction, validator: Validator = None)`

  Various interfaces to the balanceOf method.

  Persist instance data.

  Expand source code

      class BalanceOfMethod(ContractMethod): # pylint: disable=invalid-name
          """Various interfaces to the balanceOf method."""

          def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
              """Persist instance data."""
              super().__init__(web3_or_provider, contract_address, validator)
              self._underlying_method = contract_function

          def validate_and_normalize_inputs(self, account: str):
              """Validate the inputs to the balanceOf method."""
              self.validator.assert_valid(
                  method_name='balanceOf',
                  parameter_name='account',
                  argument_value=account,
              )
              account = self.validate_and_checksum_address(account)
              return (account)

          def block_call(self, account: str, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
              """Execute underlying contract method via eth_call.

              :param tx_params: transaction parameters

              """
              (account) = self.validate_and_normalize_inputs(account)

              tx_params: Optional[TxParams] = None
              tx_params = super().normalize_tx_params(tx_params)
              returned = self._underlying_method(account).call(tx_params.as_dict())
              return int(returned)

          def estimate_gas(self, account: str, tx_params: Optional[TxParams] = None) -> int:
              """Estimate gas consumption of method call."""
              (account) = self.validate_and_normalize_inputs(account)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(account).estimateGas(tx_params.as_dict())

  ### Ancestors

  - ContractMethod

  ### Methods

  `def block_call(self, account: str, val: int = 0, fee: int = 1000000, debug: bool = False) ‑> int`

    Execute underlying contract method via eth_call.

    :param tx_params: transaction parameters

    Expand source code

        def block_call(self, account: str, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters

            """
            (account) = self.validate_and_normalize_inputs(account)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(account).call(tx_params.as_dict())
            return int(returned)

  `def estimate_gas(self, account: str, tx_params: Union[TxParams, NoneType] = None) ‑> int`

    Estimate gas consumption of method call.

    Expand source code

        def estimate_gas(self, account: str, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (account) = self.validate_and_normalize_inputs(account)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(account).estimateGas(tx_params.as_dict())

  `def validate_and_normalize_inputs(self, account: str)`

    Validate the inputs to the balanceOf method.

    Expand source code

        def validate_and_normalize_inputs(self, account: str):
            """Validate the inputs to the balanceOf method."""
            self.validator.assert_valid(
                method_name='balanceOf',
                parameter_name='account',
                argument_value=account,
            )
            account = self.validate_and_checksum_address(account)
            return (account)

  ### Inherited members

  - `ContractMethod`:
    - `normalize_tx_params`
    - `validate_and_checksum_address`
`class DecreaseAllowanceMethod (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, contract_function: web3.contract.ContractFunction, validator: Validator = None)`

  Various interfaces to the decreaseAllowance method.

  Persist instance data.

  Expand source code

      class DecreaseAllowanceMethod(ContractMethod): # pylint: disable=invalid-name
          """Various interfaces to the decreaseAllowance method."""

          def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
              """Persist instance data."""
              super().__init__(web3_or_provider, contract_address, validator)
              self._underlying_method = contract_function

          def validate_and_normalize_inputs(self, spender: str, subtracted_value: int):
              """Validate the inputs to the decreaseAllowance method."""
              self.validator.assert_valid(
                  method_name='decreaseAllowance',
                  parameter_name='spender',
                  argument_value=spender,
              )
              spender = self.validate_and_checksum_address(spender)
              self.validator.assert_valid(
                  method_name='decreaseAllowance',
                  parameter_name='subtractedValue',
                  argument_value=subtracted_value,
              )
              # safeguard against fractional inputs
              subtracted_value = int(subtracted_value)
              return (spender, subtracted_value)

          def block_call(self, spender: str, subtracted_value: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
              """Execute underlying contract method via eth_call.

              :param tx_params: transaction parameters
              :returns: the return value of the underlying method.
              """
              (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)

              tx_params: Optional[TxParams] = None
              tx_params = super().normalize_tx_params(tx_params)
              returned = self._underlying_method(spender, subtracted_value).call(tx_params.as_dict())
              return bool(returned)

          def send_transaction(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
              """Execute underlying contract method via eth_sendTransaction.

              :param tx_params: transaction parameters
              """
              (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, subtracted_value).transact(tx_params.as_dict())

          def build_transaction(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> dict:
              """Construct calldata to be used as input to the method."""
              (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, subtracted_value).buildTransaction(tx_params.as_dict())

          def estimate_gas(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> int:
              """Estimate gas consumption of method call."""
              (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, subtracted_value).estimateGas(tx_params.as_dict())

  ### Ancestors

  - ContractMethod

  ### Methods

  `def block_call(self, spender: str, subtracted_value: int, val: int = 0, fee: int = 1000000, debug: bool = False) ‑> bool`

    Execute underlying contract method via eth_call.

    :param tx_params: transaction parameters :returns: the return value of the underlying method.

    Expand source code

        def block_call(self, spender: str, subtracted_value: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(spender, subtracted_value).call(tx_params.as_dict())
            return bool(returned)

  `def build_transaction(self, spender: str, subtracted_value: int, tx_params: Union[TxParams, NoneType] = None) ‑> dict`

    Construct calldata to be used as input to the method.

    Expand source code

        def build_transaction(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, subtracted_value).buildTransaction(tx_params.as_dict())

  `def estimate_gas(self, spender: str, subtracted_value: int, tx_params: Union[TxParams, NoneType] = None) ‑> int`

    Estimate gas consumption of method call.

    Expand source code

        def estimate_gas(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, subtracted_value).estimateGas(tx_params.as_dict())

  `def send_transaction(self, spender: str, subtracted_value: int, tx_params: Union[TxParams, NoneType] = None) ‑> Union[hexbytes.main.HexBytes, bytes]`

    Execute underlying contract method via eth_sendTransaction.

    :param tx_params: transaction parameters

    Expand source code

        def send_transaction(self, spender: str, subtracted_value: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (spender, subtracted_value) = self.validate_and_normalize_inputs(spender, subtracted_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, subtracted_value).transact(tx_params.as_dict())

  `def validate_and_normalize_inputs(self, spender: str, subtracted_value: int)`

    Validate the inputs to the decreaseAllowance method.

    Expand source code

        def validate_and_normalize_inputs(self, spender: str, subtracted_value: int):
            """Validate the inputs to the decreaseAllowance method."""
            self.validator.assert_valid(
                method_name='decreaseAllowance',
                parameter_name='spender',
                argument_value=spender,
            )
            spender = self.validate_and_checksum_address(spender)
            self.validator.assert_valid(
                method_name='decreaseAllowance',
                parameter_name='subtractedValue',
                argument_value=subtracted_value,
            )
            # safeguard against fractional inputs
            subtracted_value = int(subtracted_value)
            return (spender, subtracted_value)

  ### Inherited members

  - `ContractMethod`:
    - `normalize_tx_params`
    - `validate_and_checksum_address`
`class ERC20 (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, validator: ERC20Validator = None)`

  Wrapper class for ERC20 Solidity contract.

  Get an instance of wrapper for smart contract.

  :param web3_or_provider: Either an instance of `web3.Web3`:code: or `web3.providers.base.BaseProvider`:code: :param contract_address: where the contract has been deployed :param validator: for validation of method inputs.

  Expand source code

      class ERC20:
          """Wrapper class for ERC20 Solidity contract."""
          _fn_allowance: AllowanceMethod
          """Constructor-initialized instance of
          :class:`AllowanceMethod`.
          """

          _fn_approve: ApproveMethod
          """Constructor-initialized instance of
          :class:`ApproveMethod`.
          """

          _fn_balance_of: BalanceOfMethod
          """Constructor-initialized instance of
          :class:`BalanceOfMethod`.
          """

          _fn_decrease_allowance: DecreaseAllowanceMethod
          """Constructor-initialized instance of
          :class:`DecreaseAllowanceMethod`.
          """

          _fn_increase_allowance: IncreaseAllowanceMethod
          """Constructor-initialized instance of
          :class:`IncreaseAllowanceMethod`.
          """

          _fn_total_supply: TotalSupplyMethod
          """Constructor-initialized instance of
          :class:`TotalSupplyMethod`.
          """

          _fn_transfer: TransferMethod
          """Constructor-initialized instance of
          :class:`TransferMethod`.
          """

          _fn_transfer_from: TransferFromMethod
          """Constructor-initialized instance of
          :class:`TransferFromMethod`.
          """

          def __init__(
              self,
              web3_or_provider: Union[Web3, BaseProvider],
              contract_address: str,
              validator: ERC20Validator = None,
          ):
              """Get an instance of wrapper for smart contract.

              :param web3_or_provider: Either an instance of `web3.Web3`:code: or
                  `web3.providers.base.BaseProvider`:code:
              :param contract_address: where the contract has been deployed
              :param validator: for validation of method inputs.
              """
              # pylint: disable=too-many-statements

              self.contract_address = contract_address

              if not validator:
                  validator = ERC20Validator(web3_or_provider, contract_address)

              web3 = None
              if isinstance(web3_or_provider, BaseProvider):
                  web3 = Web3(web3_or_provider)
              elif isinstance(web3_or_provider, Web3):
                  web3 = web3_or_provider
              else:
                  raise TypeError(
                      "Expected parameter 'web3_or_provider' to be an instance of either"
                      + " Web3 or BaseProvider"
                  )

              # if any middleware was imported, inject it
              try:
                  MIDDLEWARE
              except NameError:
                  pass
              else:
                  try:
                      for middleware in MIDDLEWARE:
                          web3.middleware_onion.inject(
                               middleware['function'], layer=middleware['layer'],
                          )
                  except ValueError as value_error:
                      if value_error.args == ("You can't add the same un-named instance twice",):
                          pass

              self._web3_eth = web3.eth

              functions = self._web3_eth.contract(address=to_checksum_address(contract_address), abi=ERC20.abi()).functions

              self.call_contract_fee_amount:int = 1000000
              self.call_contract_debug_flag:bool = False

              self._fn_allowance = AllowanceMethod(web3_or_provider, contract_address, functions.allowance, validator)

              self._fn_approve = ApproveMethod(web3_or_provider, contract_address, functions.approve, validator)

              self._fn_balance_of = BalanceOfMethod(web3_or_provider, contract_address, functions.balanceOf, validator)

              self._fn_decrease_allowance = DecreaseAllowanceMethod(web3_or_provider, contract_address, functions.decreaseAllowance, validator)

              self._fn_increase_allowance = IncreaseAllowanceMethod(web3_or_provider, contract_address, functions.increaseAllowance, validator)

              self._fn_total_supply = TotalSupplyMethod(web3_or_provider, contract_address, functions.totalSupply)

              self._fn_transfer = TransferMethod(web3_or_provider, contract_address, functions.transfer, validator)

              self._fn_transfer_from = TransferFromMethod(web3_or_provider, contract_address, functions.transferFrom, validator)

          """
          Implementation of event approval in contract ERC20
          Get log entry for Approval event.
                  :param tx_hash: hash of transaction emitting Approval event
          """

          def event_approval(
                  self, tx_hash: Union[HexBytes, bytes]
          ) -> Tuple[AttributeDict]:
              tx_receipt = self._web3_eth.getTransactionReceipt(tx_hash)
              return self._web3_eth.contract(address=to_checksum_address(self.contract_address), abi=ERC20.abi()).events.Approval().processReceipt(tx_receipt)
          """
          Implementation of event transfer in contract ERC20
          Get log entry for Transfer event.
                  :param tx_hash: hash of transaction emitting Transfer event
          """

          def event_transfer(
                  self, tx_hash: Union[HexBytes, bytes]
          ) -> Tuple[AttributeDict]:
              tx_receipt = self._web3_eth.getTransactionReceipt(tx_hash)
              return self._web3_eth.contract(address=to_checksum_address(self.contract_address), abi=ERC20.abi()).events.Transfer().processReceipt(tx_receipt)

          """
          implementation of allowance in contract ERC20

          """

          def allowance(self,owner: str, spender: str) -> int:
              return self._fn_allowance.block_call(owner, spender)
          """
          implementation of approve in contract ERC20

          """

          def approve(self,spender: str, amount: int) -> bool:
              return self._fn_approve.block_call(spender, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)
          """
          implementation of balance_of in contract ERC20

          """

          def balance_of(self,account: str) -> int:
              return self._fn_balance_of.block_call(account)
          """
          implementation of decrease_allowance in contract ERC20

          """

          def decrease_allowance(self,spender: str, subtracted_value: int) -> bool:
              return self._fn_decrease_allowance.block_call(spender, subtracted_value, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)
          """
          implementation of increase_allowance in contract ERC20

          """

          def increase_allowance(self,spender: str, added_value: int) -> bool:
              return self._fn_increase_allowance.block_call(spender, added_value, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)
          """
          implementation of total_supply in contract ERC20

          """

          def total_supply(self) -> int:
              return self._fn_total_supply.block_call()
          """
          implementation of transfer in contract ERC20

          """

          def transfer(self,recipient: str, amount: int) -> bool:
              return self._fn_transfer.block_call(recipient, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)
          """
          implementation of transfer_from in contract ERC20

          """

          def transfer_from(self,sender: str, recipient: str, amount: int) -> bool:
              return self._fn_transfer_from.block_call(sender, recipient, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)

          def CallContractFee(self, amount:int)-> "ERC20":
              self.call_contract_fee_amount = amount
              return self

          def CallDebug(self, yesno: bool) -> "ERC20":
              self.call_contract_debug_flag = yesno
              return self

          @staticmethod
          def abi():
              """Return the ABI to the underlying contract."""
              return json.loads(
                  '[{"anonymous":false,"inputs":[{"indexed":true,"internalType":"address","name":"owner","type":"address"},{"indexed":true,"internalType":"address","name":"spender","type":"address"},{"indexed":false,"internalType":"uint256","name":"value","type":"uint256"}],"name":"Approval","type":"event"},{"anonymous":false,"inputs":[{"indexed":true,"internalType":"address","name":"from","type":"address"},{"indexed":true,"internalType":"address","name":"to","type":"address"},{"indexed":false,"internalType":"uint256","name":"value","type":"uint256"}],"name":"Transfer","type":"event"},{"constant":true,"inputs":[{"internalType":"address","name":"owner","type":"address"},{"internalType":"address","name":"spender","type":"address"}],"name":"allowance","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"approve","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[{"internalType":"address","name":"account","type":"address"}],"name":"balanceOf","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"subtractedValue","type":"uint256"}],"name":"decreaseAllowance","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"addedValue","type":"uint256"}],"name":"increaseAllowance","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[],"name":"totalSupply","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"recipient","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"transfer","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"sender","type":"address"},{"internalType":"address","name":"recipient","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"transferFrom","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"}]'  # noqa: E501 (line-too-long)
              )

  ### Static methods

  `def abi()`

    Return the ABI to the underlying contract.

    Expand source code

        @staticmethod
        def abi():
            """Return the ABI to the underlying contract."""
            return json.loads(
                '[{"anonymous":false,"inputs":[{"indexed":true,"internalType":"address","name":"owner","type":"address"},{"indexed":true,"internalType":"address","name":"spender","type":"address"},{"indexed":false,"internalType":"uint256","name":"value","type":"uint256"}],"name":"Approval","type":"event"},{"anonymous":false,"inputs":[{"indexed":true,"internalType":"address","name":"from","type":"address"},{"indexed":true,"internalType":"address","name":"to","type":"address"},{"indexed":false,"internalType":"uint256","name":"value","type":"uint256"}],"name":"Transfer","type":"event"},{"constant":true,"inputs":[{"internalType":"address","name":"owner","type":"address"},{"internalType":"address","name":"spender","type":"address"}],"name":"allowance","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"approve","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[{"internalType":"address","name":"account","type":"address"}],"name":"balanceOf","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"subtractedValue","type":"uint256"}],"name":"decreaseAllowance","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"spender","type":"address"},{"internalType":"uint256","name":"addedValue","type":"uint256"}],"name":"increaseAllowance","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":true,"inputs":[],"name":"totalSupply","outputs":[{"internalType":"uint256","name":"","type":"uint256"}],"payable":false,"stateMutability":"view","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"recipient","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"transfer","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"},{"constant":false,"inputs":[{"internalType":"address","name":"sender","type":"address"},{"internalType":"address","name":"recipient","type":"address"},{"internalType":"uint256","name":"amount","type":"uint256"}],"name":"transferFrom","outputs":[{"internalType":"bool","name":"","type":"bool"}],"payable":false,"stateMutability":"nonpayable","type":"function"}]'  # noqa: E501 (line-too-long)
            )

  ### Methods

  `def CallContractFee(self, amount: int) ‑> ERC20`
    Expand source code

        def CallContractFee(self, amount:int)-> "ERC20":
            self.call_contract_fee_amount = amount
            return self

  `def CallDebug(self, yesno: bool) ‑> ERC20`
    Expand source code

        def CallDebug(self, yesno: bool) -> "ERC20":
            self.call_contract_debug_flag = yesno
            return self

  `def allowance(self, owner: str, spender: str) ‑> int`
    Expand source code

        def allowance(self,owner: str, spender: str) -> int:
            return self._fn_allowance.block_call(owner, spender)

  `def approve(self, spender: str, amount: int) ‑> bool`
    Expand source code

        def approve(self,spender: str, amount: int) -> bool:
            return self._fn_approve.block_call(spender, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)

  `def balance_of(self, account: str) ‑> int`
    Expand source code

        def balance_of(self,account: str) -> int:
            return self._fn_balance_of.block_call(account)

  `def decrease_allowance(self, spender: str, subtracted_value: int) ‑> bool`
    Expand source code

        def decrease_allowance(self,spender: str, subtracted_value: int) -> bool:
            return self._fn_decrease_allowance.block_call(spender, subtracted_value, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)

  `def event_approval(self, tx_hash: Union[hexbytes.main.HexBytes, bytes]) ‑> Tuple[web3.datastructures.AttributeDict]`
    Expand source code

        def event_approval(
                self, tx_hash: Union[HexBytes, bytes]
        ) -> Tuple[AttributeDict]:
            tx_receipt = self._web3_eth.getTransactionReceipt(tx_hash)
            return self._web3_eth.contract(address=to_checksum_address(self.contract_address), abi=ERC20.abi()).events.Approval().processReceipt(tx_receipt)

  `def event_transfer(self, tx_hash: Union[hexbytes.main.HexBytes, bytes]) ‑> Tuple[web3.datastructures.AttributeDict]`
    Expand source code

        def event_transfer(
                self, tx_hash: Union[HexBytes, bytes]
        ) -> Tuple[AttributeDict]:
            tx_receipt = self._web3_eth.getTransactionReceipt(tx_hash)
            return self._web3_eth.contract(address=to_checksum_address(self.contract_address), abi=ERC20.abi()).events.Transfer().processReceipt(tx_receipt)

  `def increase_allowance(self, spender: str, added_value: int) ‑> bool`
    Expand source code

        def increase_allowance(self,spender: str, added_value: int) -> bool:
            return self._fn_increase_allowance.block_call(spender, added_value, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)

  `def total_supply(self) ‑> int`
    Expand source code

        def total_supply(self) -> int:
            return self._fn_total_supply.block_call()

  `def transfer(self, recipient: str, amount: int) ‑> bool`
    Expand source code

        def transfer(self,recipient: str, amount: int) -> bool:
            return self._fn_transfer.block_call(recipient, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)

  `def transfer_from(self, sender: str, recipient: str, amount: int) ‑> bool`
    Expand source code

        def transfer_from(self,sender: str, recipient: str, amount: int) -> bool:
            return self._fn_transfer_from.block_call(sender, recipient, amount, 0,self.call_contract_fee_amount,self.call_contract_debug_flag)

`class ERC20Validator (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str)`

  No-op input validator.

  Initialize the instance.

  Expand source code

      class ERC20Validator(  # type: ignore
          Validator
      ):
          """No-op input validator."""

  ### Ancestors

  - Validator

  ### Inherited members

  - `Validator`:
    - `assert_valid`
`class IncreaseAllowanceMethod (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, contract_function: web3.contract.ContractFunction, validator: Validator = None)`

  Various interfaces to the increaseAllowance method.

  Persist instance data.

  Expand source code

      class IncreaseAllowanceMethod(ContractMethod): # pylint: disable=invalid-name
          """Various interfaces to the increaseAllowance method."""

          def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
              """Persist instance data."""
              super().__init__(web3_or_provider, contract_address, validator)
              self._underlying_method = contract_function

          def validate_and_normalize_inputs(self, spender: str, added_value: int):
              """Validate the inputs to the increaseAllowance method."""
              self.validator.assert_valid(
                  method_name='increaseAllowance',
                  parameter_name='spender',
                  argument_value=spender,
              )
              spender = self.validate_and_checksum_address(spender)
              self.validator.assert_valid(
                  method_name='increaseAllowance',
                  parameter_name='addedValue',
                  argument_value=added_value,
              )
              # safeguard against fractional inputs
              added_value = int(added_value)
              return (spender, added_value)

          def block_call(self, spender: str, added_value: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
              """Execute underlying contract method via eth_call.

              :param tx_params: transaction parameters
              :returns: the return value of the underlying method.
              """
              (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)

              tx_params: Optional[TxParams] = None
              tx_params = super().normalize_tx_params(tx_params)
              returned = self._underlying_method(spender, added_value).call(tx_params.as_dict())
              return bool(returned)

          def send_transaction(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
              """Execute underlying contract method via eth_sendTransaction.

              :param tx_params: transaction parameters
              """
              (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, added_value).transact(tx_params.as_dict())

          def build_transaction(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> dict:
              """Construct calldata to be used as input to the method."""
              (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, added_value).buildTransaction(tx_params.as_dict())

          def estimate_gas(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> int:
              """Estimate gas consumption of method call."""
              (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(spender, added_value).estimateGas(tx_params.as_dict())

  ### Ancestors

  - ContractMethod

  ### Methods

  `def block_call(self, spender: str, added_value: int, val: int = 0, fee: int = 1000000, debug: bool = False) ‑> bool`

    Execute underlying contract method via eth_call.

    :param tx_params: transaction parameters :returns: the return value of the underlying method.

    Expand source code

        def block_call(self, spender: str, added_value: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(spender, added_value).call(tx_params.as_dict())
            return bool(returned)

  `def build_transaction(self, spender: str, added_value: int, tx_params: Union[TxParams, NoneType] = None) ‑> dict`

    Construct calldata to be used as input to the method.

    Expand source code

        def build_transaction(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, added_value).buildTransaction(tx_params.as_dict())

  `def estimate_gas(self, spender: str, added_value: int, tx_params: Union[TxParams, NoneType] = None) ‑> int`

    Estimate gas consumption of method call.

    Expand source code

        def estimate_gas(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, added_value).estimateGas(tx_params.as_dict())

  `def send_transaction(self, spender: str, added_value: int, tx_params: Union[TxParams, NoneType] = None) ‑> Union[hexbytes.main.HexBytes, bytes]`

    Execute underlying contract method via eth_sendTransaction.

    :param tx_params: transaction parameters

    Expand source code

        def send_transaction(self, spender: str, added_value: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (spender, added_value) = self.validate_and_normalize_inputs(spender, added_value)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(spender, added_value).transact(tx_params.as_dict())

  `def validate_and_normalize_inputs(self, spender: str, added_value: int)`

    Validate the inputs to the increaseAllowance method.

    Expand source code

        def validate_and_normalize_inputs(self, spender: str, added_value: int):
            """Validate the inputs to the increaseAllowance method."""
            self.validator.assert_valid(
                method_name='increaseAllowance',
                parameter_name='spender',
                argument_value=spender,
            )
            spender = self.validate_and_checksum_address(spender)
            self.validator.assert_valid(
                method_name='increaseAllowance',
                parameter_name='addedValue',
                argument_value=added_value,
            )
            # safeguard against fractional inputs
            added_value = int(added_value)
            return (spender, added_value)

  ### Inherited members

  - `ContractMethod`:
    - `normalize_tx_params`
    - `validate_and_checksum_address`
`class TotalSupplyMethod (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, contract_function: web3.contract.ContractFunction)`

  Various interfaces to the totalSupply method.

  Persist instance data.

  Expand source code

      class TotalSupplyMethod(ContractMethod): # pylint: disable=invalid-name
          """Various interfaces to the totalSupply method."""

          def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction):
              """Persist instance data."""
              super().__init__(web3_or_provider, contract_address)
              self._underlying_method = contract_function

          def block_call(self, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
              """Execute underlying contract method via eth_call.

              :param tx_params: transaction parameters

              """

              tx_params: Optional[TxParams] = None
              tx_params = super().normalize_tx_params(tx_params)
              returned = self._underlying_method().call(tx_params.as_dict())
              return int(returned)

          def estimate_gas(self, tx_params: Optional[TxParams] = None) -> int:
              """Estimate gas consumption of method call."""
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method().estimateGas(tx_params.as_dict())

  ### Ancestors

  - ContractMethod

  ### Methods

  `def block_call(self, val: int = 0, fee: int = 1000000, debug: bool = False) ‑> int`

    Execute underlying contract method via eth_call.

    :param tx_params: transaction parameters

    Expand source code

        def block_call(self, val: int = 0, fee: int = 1000000, debug: bool = False) -> int:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters

            """

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method().call(tx_params.as_dict())
            return int(returned)

  `def estimate_gas(self, tx_params: Union[TxParams, NoneType] = None) ‑> int`

    Estimate gas consumption of method call.

    Expand source code

        def estimate_gas(self, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method().estimateGas(tx_params.as_dict())

  ### Inherited members

  - `ContractMethod`:
    - `normalize_tx_params`
    - `validate_and_checksum_address`
`class TransferFromMethod (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, contract_function: web3.contract.ContractFunction, validator: Validator = None)`

  Various interfaces to the transferFrom method.

  Persist instance data.

  Expand source code

      class TransferFromMethod(ContractMethod): # pylint: disable=invalid-name
          """Various interfaces to the transferFrom method."""

          def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
              """Persist instance data."""
              super().__init__(web3_or_provider, contract_address, validator)
              self._underlying_method = contract_function

          def validate_and_normalize_inputs(self, sender: str, recipient: str, amount: int):
              """Validate the inputs to the transferFrom method."""
              self.validator.assert_valid(
                  method_name='transferFrom',
                  parameter_name='sender',
                  argument_value=sender,
              )
              sender = self.validate_and_checksum_address(sender)
              self.validator.assert_valid(
                  method_name='transferFrom',
                  parameter_name='recipient',
                  argument_value=recipient,
              )
              recipient = self.validate_and_checksum_address(recipient)
              self.validator.assert_valid(
                  method_name='transferFrom',
                  parameter_name='amount',
                  argument_value=amount,
              )
              # safeguard against fractional inputs
              amount = int(amount)
              return (sender, recipient, amount)

          def block_call(self, sender: str, recipient: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
              """Execute underlying contract method via eth_call.

              :param tx_params: transaction parameters
              :returns: the return value of the underlying method.
              """
              (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)

              tx_params: Optional[TxParams] = None
              tx_params = super().normalize_tx_params(tx_params)
              returned = self._underlying_method(sender, recipient, amount).call(tx_params.as_dict())
              return bool(returned)

          def send_transaction(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
              """Execute underlying contract method via eth_sendTransaction.

              :param tx_params: transaction parameters
              """
              (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(sender, recipient, amount).transact(tx_params.as_dict())

          def build_transaction(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
              """Construct calldata to be used as input to the method."""
              (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(sender, recipient, amount).buildTransaction(tx_params.as_dict())

          def estimate_gas(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
              """Estimate gas consumption of method call."""
              (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(sender, recipient, amount).estimateGas(tx_params.as_dict())

  ### Ancestors

  - ContractMethod

  ### Methods

  `def block_call(self, sender: str, recipient: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) ‑> bool`

    Execute underlying contract method via eth_call.

    :param tx_params: transaction parameters :returns: the return value of the underlying method.

    Expand source code

        def block_call(self, sender: str, recipient: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(sender, recipient, amount).call(tx_params.as_dict())
            return bool(returned)

  `def build_transaction(self, sender: str, recipient: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> dict`

    Construct calldata to be used as input to the method.

    Expand source code

        def build_transaction(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(sender, recipient, amount).buildTransaction(tx_params.as_dict())

  `def estimate_gas(self, sender: str, recipient: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> int`

    Estimate gas consumption of method call.

    Expand source code

        def estimate_gas(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(sender, recipient, amount).estimateGas(tx_params.as_dict())

  `def send_transaction(self, sender: str, recipient: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> Union[hexbytes.main.HexBytes, bytes]`

    Execute underlying contract method via eth_sendTransaction.

    :param tx_params: transaction parameters

    Expand source code

        def send_transaction(self, sender: str, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (sender, recipient, amount) = self.validate_and_normalize_inputs(sender, recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(sender, recipient, amount).transact(tx_params.as_dict())

  `def validate_and_normalize_inputs(self, sender: str, recipient: str, amount: int)`

    Validate the inputs to the transferFrom method.

    Expand source code

        def validate_and_normalize_inputs(self, sender: str, recipient: str, amount: int):
            """Validate the inputs to the transferFrom method."""
            self.validator.assert_valid(
                method_name='transferFrom',
                parameter_name='sender',
                argument_value=sender,
            )
            sender = self.validate_and_checksum_address(sender)
            self.validator.assert_valid(
                method_name='transferFrom',
                parameter_name='recipient',
                argument_value=recipient,
            )
            recipient = self.validate_and_checksum_address(recipient)
            self.validator.assert_valid(
                method_name='transferFrom',
                parameter_name='amount',
                argument_value=amount,
            )
            # safeguard against fractional inputs
            amount = int(amount)
            return (sender, recipient, amount)

  ### Inherited members

  - `ContractMethod`:
    - `normalize_tx_params`
    - `validate_and_checksum_address`
`class TransferMethod (web3_or_provider: Union[web3.main.Web3, web3.providers.base.BaseProvider], contract_address: str, contract_function: web3.contract.ContractFunction, validator: Validator = None)`

  Various interfaces to the transfer method.

  Persist instance data.

  Expand source code

      class TransferMethod(ContractMethod): # pylint: disable=invalid-name
          """Various interfaces to the transfer method."""

          def __init__(self, web3_or_provider: Union[Web3, BaseProvider], contract_address: str, contract_function: ContractFunction, validator: Validator=None):
              """Persist instance data."""
              super().__init__(web3_or_provider, contract_address, validator)
              self._underlying_method = contract_function

          def validate_and_normalize_inputs(self, recipient: str, amount: int):
              """Validate the inputs to the transfer method."""
              self.validator.assert_valid(
                  method_name='transfer',
                  parameter_name='recipient',
                  argument_value=recipient,
              )
              recipient = self.validate_and_checksum_address(recipient)
              self.validator.assert_valid(
                  method_name='transfer',
                  parameter_name='amount',
                  argument_value=amount,
              )
              # safeguard against fractional inputs
              amount = int(amount)
              return (recipient, amount)

          def block_call(self, recipient: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
              """Execute underlying contract method via eth_call.

              :param tx_params: transaction parameters
              :returns: the return value of the underlying method.
              """
              (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)

              tx_params: Optional[TxParams] = None
              tx_params = super().normalize_tx_params(tx_params)
              returned = self._underlying_method(recipient, amount).call(tx_params.as_dict())
              return bool(returned)

          def send_transaction(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
              """Execute underlying contract method via eth_sendTransaction.

              :param tx_params: transaction parameters
              """
              (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(recipient, amount).transact(tx_params.as_dict())

          def build_transaction(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
              """Construct calldata to be used as input to the method."""
              (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(recipient, amount).buildTransaction(tx_params.as_dict())

          def estimate_gas(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
              """Estimate gas consumption of method call."""
              (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
              tx_params = super().normalize_tx_params(tx_params)
              return self._underlying_method(recipient, amount).estimateGas(tx_params.as_dict())

  ### Ancestors

  - ContractMethod

  ### Methods

  `def block_call(self, recipient: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) ‑> bool`

    Execute underlying contract method via eth_call.

    :param tx_params: transaction parameters :returns: the return value of the underlying method.

    Expand source code

        def block_call(self, recipient: str, amount: int, val: int = 0, fee: int = 1000000, debug: bool = False) -> bool:
            """Execute underlying contract method via eth_call.

            :param tx_params: transaction parameters
            :returns: the return value of the underlying method.
            """
            (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)

            tx_params: Optional[TxParams] = None
            tx_params = super().normalize_tx_params(tx_params)
            returned = self._underlying_method(recipient, amount).call(tx_params.as_dict())
            return bool(returned)

  `def build_transaction(self, recipient: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> dict`

    Construct calldata to be used as input to the method.

    Expand source code

        def build_transaction(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> dict:
            """Construct calldata to be used as input to the method."""
            (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(recipient, amount).buildTransaction(tx_params.as_dict())

  `def estimate_gas(self, recipient: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> int`

    Estimate gas consumption of method call.

    Expand source code

        def estimate_gas(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> int:
            """Estimate gas consumption of method call."""
            (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(recipient, amount).estimateGas(tx_params.as_dict())

  `def send_transaction(self, recipient: str, amount: int, tx_params: Union[TxParams, NoneType] = None) ‑> Union[hexbytes.main.HexBytes, bytes]`

    Execute underlying contract method via eth_sendTransaction.

    :param tx_params: transaction parameters

    Expand source code

        def send_transaction(self, recipient: str, amount: int, tx_params: Optional[TxParams] = None) -> Union[HexBytes, bytes]:
            """Execute underlying contract method via eth_sendTransaction.

            :param tx_params: transaction parameters
            """
            (recipient, amount) = self.validate_and_normalize_inputs(recipient, amount)
            tx_params = super().normalize_tx_params(tx_params)
            return self._underlying_method(recipient, amount).transact(tx_params.as_dict())

  `def validate_and_normalize_inputs(self, recipient: str, amount: int)`

    Validate the inputs to the transfer method.

    Expand source code

        def validate_and_normalize_inputs(self, recipient: str, amount: int):
            """Validate the inputs to the transfer method."""
            self.validator.assert_valid(
                method_name='transfer',
                parameter_name='recipient',
                argument_value=recipient,
            )
            recipient = self.validate_and_checksum_address(recipient)
            self.validator.assert_valid(
                method_name='transfer',
                parameter_name='amount',
                argument_value=amount,
            )
            # safeguard against fractional inputs
            amount = int(amount)
            return (recipient, amount)

  ### Inherited members

  - `ContractMethod`:
    - `normalize_tx_params`
    - `validate_and_checksum_address`